@@ -2,11 +2,11 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: main.c,v 1.200 2004/04/18 02:16:05 mast Exp $
+|| $Id: main.c,v 1.205 2004/12/17 17:27:25 grubba Exp $
 */
 
 #include "global.h"
-RCSID("$Id: main.c,v 1.200 2004/04/18 02:16:05 mast Exp $");
+RCSID("$Id: main.c,v 1.205 2004/12/17 17:27:25 grubba Exp $");
 #include "fdlib.h"
 #include "backend.h"
 #include "module.h"
@@ -177,7 +177,9 @@ static void get_master_key(HKEY cat)
 		  (LPCTSTR)("SOFTWARE\\Pike\\"
 			    DEFINETOSTR(PIKE_MAJOR_VERSION)
 			    "."
-			    DEFINETOSTR(PIKE_MINOR_VERSION)),
+			    DEFINETOSTR(PIKE_MINOR_VERSION)
+			    "."
+			    DEFINETOSTR(PIKE_BUILD_VERSION)),
 		  0,KEY_READ,&k)==ERROR_SUCCESS)
   {
     if(RegQueryValueEx(k,
@@ -282,7 +284,7 @@ int dbm_main(int argc, char **argv)
     init_pike_frame_blocks();
     init_node_s_blocks();
     init_object_blocks();
-#ifndef DEBUG_MALLOC
+#if !defined(DEBUG_MALLOC) || !defined(_REENTRANT)
     /* This has already been done by initialize_dmalloc(). */
     init_callback_blocks();
 #endif /* !DEBUG_MALLOC */
@@ -335,7 +337,7 @@ int dbm_main(int argc, char **argv)
   if(master_location[CONSTANT_STRLEN(MASTER_COOKIE)])
     master_file=master_location + CONSTANT_STRLEN(MASTER_COOKIE);
 
-#if __NT__
+#ifdef __NT__
   if(!master_file) get_master_key(HKEY_CURRENT_USER);
   if(!master_file) get_master_key(HKEY_LOCAL_MACHINE);
 #endif
@@ -815,7 +817,7 @@ int dbm_main(int argc, char **argv)
     
 #ifdef __amigaos__
 #ifdef __amigaos4__
-    if(DOSBase->dl_lib.lib_Version >= 50) {
+    if(DOSBase->lib_Version >= 50) {
       struct svalue *mark = Pike_sp;
       IDOS->ScanVars(&scan_amigaos_environment_hook,
 		     GVF_BINARY_VAR|GVF_DONT_NULL_TERM,
@@ -916,27 +918,24 @@ void exit_main(void)
 #ifdef DO_PIKE_CLEANUP
   size_t count;
 
-#ifdef PIKE_DEBUG
-  if (verbose_debug_exit)
-    fprintf(stderr,"Exited normally, counting bytes.\n");
-#endif
-
-  /* Destruct all remaining objects while we have a proper execution
-   * environment. The downside is that the leak report below will
-   * always report destructed objects. We use the gc in a special mode
-   * for this to get a reasonably sane destruct order. */
-  gc_destruct_everything = 1;
-  count = do_gc (NULL, 1);
-  while (count) {
-    size_t new_count = do_gc (NULL, 1);
-    if (new_count >= count) {
-      fprintf (stderr, "Some destroy function is creating new objects "
-	       "during final cleanup - can't exit cleanly.\n");
-      break;
+  if (exit_with_cleanup) {
+    /* Destruct all remaining objects while we have a proper execution
+     * environment. The downside is that the leak report below will
+     * always report destructed objects. We use the gc in a special mode
+     * for this to get a reasonably sane destruct order. */
+    gc_destruct_everything = 1;
+    count = do_gc (NULL, 1);
+    while (count) {
+      size_t new_count = do_gc (NULL, 1);
+      if (new_count >= count) {
+	fprintf (stderr, "Some destroy function is creating new objects "
+		 "during final cleanup - can't exit cleanly.\n");
+	break;
+      }
+      count = new_count;
     }
-    count = new_count;
+    gc_destruct_everything = 0;
   }
-  gc_destruct_everything = 0;
 
   /* Unload dynamic modules before static ones. */
   exit_dynamic_load();
@@ -988,14 +987,15 @@ void low_exit_main(void)
 
   do_gc(NULL, 1);
 
-#ifdef PIKE_DEBUG
-  if(verbose_debug_exit)
+  if (exit_with_cleanup)
   {
+    int leak_found = 0;
+
 #ifdef _REENTRANT
     if(count_pike_threads()>1)
     {
       fprintf(stderr,"Byte counting aborted, because all threads have not exited properly.\n");
-      verbose_debug_exit=0;
+      exit_with_cleanup = 0;
       return;
     }
 #endif
@@ -1014,7 +1014,6 @@ void low_exit_main(void)
 #define STATIC_ARRAYS {&empty_array, &weak_empty_array, &weak_shrink_empty_array}
 
 #define REPORT_LINKED_LIST_LEAKS(TYPE, START, STATICS, T_TYPE, NAME) do { \
-      size_t num = 0;							\
       struct TYPE *x;							\
       for (x = START; x; x = x->next) {					\
 	struct marker *m = find_marker (x);				\
@@ -1032,15 +1031,18 @@ void low_exit_main(void)
 	    if (x == statics[i])					\
 	      is_static = 1;						\
 	  if (x->refs != m->refs + is_static) {				\
-	    num++;							\
-	    fprintf (stderr, NAME " got %d unaccounted references:\n",	\
+	    if (!leak_found) {						\
+	      fputs ("Leak(s) found at exit:\n", stderr);		\
+	      leak_found = 1;						\
+	    }								\
+	    fprintf (stderr, NAME " got %d unaccounted references: ",	\
 		     x->refs - (m->refs + is_static));			\
-	    describe_something (x, T_TYPE, 2, 2, 0, NULL);		\
+	    print_short_svalue (stderr, (union anything *) &x, T_TYPE);	\
+	    fputc ('\n', stderr);					\
+	    DO_IF_DMALLOC (debug_malloc_dump_references (x, 0, 1, 0));	\
 	  }								\
 	}								\
       }									\
-      if (num)								\
-	fprintf (stderr, NAME "s left: %"PRINTSIZET"d\n", num);		\
     } while (0)
 
     REPORT_LINKED_LIST_LEAKS (array, first_array, STATIC_ARRAYS, T_ARRAY, "Array");
@@ -1056,19 +1058,23 @@ void low_exit_main(void)
      * level dmalloc reports. */
 
 #define ZAP_LINKED_LIST_LEAKS(TYPE, START, STATICS) do {		\
-      struct TYPE *x;							\
-      for (x = START; x; x = x->next) {					\
+      struct TYPE *x, *next;						\
+      for (x = START; x; x = next) {					\
 	struct marker *m = find_marker (x);				\
+	next = x->next;							\
 	if (m) {							\
 	  int is_static = 0;						\
 	  static const struct TYPE *statics[] = STATICS;		\
 	  ptrdiff_t i; /* Use signed type to avoid warnings from gcc. */ \
+	  INT32 refs;							\
 	  for (i = 0; i < (ptrdiff_t) NELEM (statics); i++)		\
 	    if (x == statics[i])					\
 	      is_static = 1;						\
-	  while (x->refs > m->refs + is_static) {			\
-	    m->flags |= GC_CLEANUP_FREED;				\
+	  refs = x->refs;						\
+	  while (refs > m->refs + is_static) {				\
+	    DO_IF_DEBUG (m->flags |= GC_CLEANUP_FREED);			\
 	    PIKE_CONCAT(free_, TYPE) (x);				\
+	    refs--;							\
 	  }								\
 	}								\
       }									\
@@ -1082,9 +1088,11 @@ void low_exit_main(void)
 
 #undef ZAP_LINKED_LIST_LEAKS
 
+#ifdef PIKE_DEBUG
     /* If we stumble on the real refs whose refcounts we've zapped
      * above we should try to handle it gracefully. */
     gc_external_refs_zapped = 1;
+#endif
 
     do_gc (NULL, 1);
 
@@ -1101,7 +1109,6 @@ void low_exit_main(void)
     }
 #endif
   }
-#endif
 
   destruct_objects_to_destruct_cb();
 
