@@ -1,13 +1,15 @@
-/*\
-||| This file a part of Pike, and is copyright by Fredrik Hubinette
-||| Pike is distributed as GPL (General Public License)
-||| See the files COPYING and DISCLAIMER for more information.
-\*/
-/**/
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: zlibmod.c,v 1.66 2004/03/19 14:36:24 nilsson Exp $
+*/
+
 #include "global.h"
-RCSID("$Id: zlibmod.c,v 1.41 2002/10/29 09:26:15 grubba Exp $");
+RCSID("$Id: zlibmod.c,v 1.66 2004/03/19 14:36:24 nilsson Exp $");
 
 #include "zlib_machine.h"
+#include "module.h"
 
 #if !defined(HAVE_LIBZ) && !defined(HAVE_LIBGZ)
 #undef HAVE_ZLIB_H
@@ -26,21 +28,19 @@ RCSID("$Id: zlibmod.c,v 1.41 2002/10/29 09:26:15 grubba Exp $");
 #include "pike_types.h"
 #include "threads.h"
 #include "dynamic_buffer.h"
+#include "operators.h"
 
 #include <zlib.h>
 
-#endif /* HAVE_ZLIB_H */
 
-/* This must be included last! */
-#include "module_magic.h"
-
-#ifdef HAVE_ZLIB_H
+#define sp Pike_sp
 
 struct zipper
 {
-  char level;
-  char state;
+  int  level;
+  int  state;
   struct z_stream_s gz;
+  struct pike_string *epilogue;
 #ifdef _REENTRANT
   DEFINE_MUTEX(lock);
 #endif /* _REENTRANT */
@@ -73,23 +73,38 @@ struct zipper
  */
 
 /*! @class deflate
+ *!
+ *! Gz_deflate is a builtin program written in C. It interfaces the
+ *! packing routines in the libz library.
+ *!
+ *! @note
+ *! This program is only available if libz was available and found when
+ *! Pike was compiled.
  *!
  *! @seealso
  *! @[Gz.inflate()]
  */
 
-/*! @decl void create(int(0..9)|void X)
+/*! @decl void create(int(0..9)|void level, int|void strategy)
  *!
- *! If given, @[X] should be a number from 0 to 9 indicating the
+ *! If given, @[level] should be a number from 0 to 9 indicating the
  *! packing / CPU ratio. Zero means no packing, 2-3 is considered 'fast',
  *! 6 is default and higher is considered 'slow' but gives better packing.
  *!
  *! This function can also be used to re-initialize a Gz.deflate object
  *! so it can be re-used.
+ *!
+ *! If the argument is negative, no headers will be emitted. This is
+ *! needed to produce ZIP-files, as an example. The negative value is
+ *! then negated, and handled as a positive value.
+ *!
+ *! @[strategy], if given, should be one of DEFAULT_STRATEGY, FILTERED or
+ *! HUFFMAN_ONLY.
  */
 static void gz_deflate_create(INT32 args)
 {
-  int tmp;
+  int tmp, wbits = 15;
+  int strategy = Z_DEFAULT_STRATEGY;
   THIS->level=Z_DEFAULT_COMPRESSION;
 
   if(THIS->gz.state)
@@ -104,6 +119,11 @@ static void gz_deflate_create(INT32 args)
     if(sp[-args].type != T_INT)
       Pike_error("Bad argument 1 to gz->create()\n");
     THIS->level=sp[-args].u.integer;
+    if( THIS->level < 0 )
+    {
+      wbits = -wbits;
+      THIS->level = -THIS->level;
+    }
     if(THIS->level < Z_NO_COMPRESSION ||
        THIS->level > Z_BEST_COMPRESSION)
     {
@@ -111,13 +131,25 @@ static void gz_deflate_create(INT32 args)
     }
   }
 
+  if(args>1)
+  {
+    if(sp[1-args].type != T_INT)
+      Pike_error("Bad argument 2 to gz->create()\n");
+    strategy=sp[1-args].u.integer;
+    if(strategy != Z_DEFAULT_STRATEGY && strategy != Z_FILTERED &&
+       strategy != Z_HUFFMAN_ONLY)
+    {
+      Pike_error("Invalid compression strategy for gz_deflate->create()\n");
+    }
+  }
+
   THIS->gz.zalloc=Z_NULL;
   THIS->gz.zfree=Z_NULL;
   THIS->gz.opaque=(void *)THIS;
 
   pop_n_elems(args);
 /*   mt_lock(& THIS->lock); */
-  tmp=deflateInit(&THIS->gz, THIS->level);
+  tmp=deflateInit2(&THIS->gz, THIS->level, Z_DEFLATED, wbits, 9, strategy );
 /*   mt_unlock(& THIS->lock); */
   switch(tmp)
   {
@@ -150,7 +182,7 @@ static int do_deflate(dynamic_buffer *buf,
    else
       do
       {
-	 this->gz.next_out=low_make_buf_space(
+	 this->gz.next_out=(Bytef *)low_make_buf_space(
 	    /* recommended by the zlib people */
 	    (this->gz.avail_out =
 	     this->gz.avail_in ?
@@ -278,12 +310,14 @@ static void init_gz_deflate(struct object *o)
   THIS->gz.opaque=(void *)THIS;
   THIS->state=0;
   deflateInit(& THIS->gz, THIS->level = Z_DEFAULT_COMPRESSION);
+  THIS->epilogue = NULL;
 }
 
 static void exit_gz_deflate(struct object *o)
 {
 /*   mt_lock(& THIS->lock); */
   deflateEnd(&THIS->gz);
+  do_free_string(THIS->epilogue);
 /*   mt_unlock(& THIS->lock); */
   mt_destroy( & THIS->lock );
 }
@@ -294,9 +328,40 @@ static void exit_gz_deflate(struct object *o)
 /*******************************************************************/
 
 /*! @class inflate
+ *!
+ *! Gz_deflate is a builtin program written in C. It interfaces the
+ *! unpacking routines in the libz library.
+ *!
+ *! @note
+ *! This program is only available if libz was available and found when
+ *! Pike was compiled.
+ *!
+ *! @seealso
+ *!   @[deflate]
  */
 
-/*! @decl void create()
+/*! @decl void create(int|void window_size)
+ *! @param magic
+ *! The window_size value is passed down to inflateInit2 in zlib.
+ *!
+ *! If the argument is negative, no header checks are done, and no
+ *! verification of the data will be done either. This is needed for
+ *! uncompressing ZIP-files, as an example. The negative value is then
+ *! negated, and handled as a positive value.
+ *!
+ *! Positive arguments set the maximum dictionary size to an exponent
+ *! of 2, such that 8 (the minimum) will cause the window size to be
+ *! 256, and 15 (the maximum, and default value) will cause it to be
+ *! 32Kb. Setting this to anything except 15 is rather pointless in
+ *! Pike.
+ *!
+ *! It can be used to limit the amount of memory that is used to
+ *! uncompress files, but 32Kb is not all that much in the great
+ *! scheme of things.
+ *!
+ *! To decompress files compressed with level 9 compression, a 32Kb
+ *! window size is needed. level 1 compression only requires a 256
+ *! byte window.
  */
 static void gz_inflate_create(INT32 args)
 {
@@ -312,10 +377,17 @@ static void gz_inflate_create(INT32 args)
   THIS->gz.zalloc=Z_NULL;
   THIS->gz.zfree=Z_NULL;
   THIS->gz.opaque=(void *)THIS;
-
+  if( args  && Pike_sp[-1].type == PIKE_T_INT )
+  {
+    tmp=inflateInit2(& THIS->gz, Pike_sp[-1].u.integer);
+  }
+  else
+  {
+    tmp=inflateInit( &THIS->gz );
+  }
   pop_n_elems(args);
+
 /*    mt_lock(& THIS->lock);  */
-  tmp=inflateInit(& THIS->gz);
 /*    mt_unlock(& THIS->lock); */
   switch(tmp)
   {
@@ -399,6 +471,15 @@ static int do_inflate(dynamic_buffer *buf,
  *! This function performs gzip style decompression. It can inflate
  *! a whole file at once or in blocks.
  *!
+ *! @example
+ *! // whole file
+ *! write(Gz_inflate()->inflate(stdin->read(0x7fffffff));
+ *!
+ *! // streaming (blocks)
+ *! function inflate=Gz_inflate()->inflate;
+ *! while(string s=stdin->read(8192))
+ *!   write(inflate(s));
+ *!
  *! @seealso
  *! @[Gz.deflate->deflate()]
  */
@@ -442,6 +523,24 @@ static void gz_inflate(INT32 args)
   pop_n_elems(args);
 
   push_string(low_free_buf(&buf));
+
+  if(fail == Z_STREAM_END)
+  {
+    struct pike_string *old_epilogue = this->epilogue;
+    if(old_epilogue) {
+      push_string(old_epilogue);
+      this->epilogue = NULL;
+    }
+    push_string(make_shared_binary_string((const char *)this->gz.next_in,
+					  this->gz.avail_in));
+    if(old_epilogue)
+      f_add(2);
+    if(sp[-1].type == PIKE_T_STRING)
+      this->epilogue = (--sp)->u.string;
+    else
+      pop_stack();
+  }
+
   if(fail != Z_STREAM_END && fail!=Z_OK && !sp[-1].u.string->len)
   {
     pop_stack();
@@ -449,6 +548,24 @@ static void gz_inflate(INT32 args)
   }
 }
 
+/*! @decl string end_of_stream()
+ *!
+ *! This function returns 0 if the end of stream marker has not yet
+ *! been encountered, or a string (possibly empty) containg any extra data
+ *! received following the end of stream marker if the marker has been
+ *! encountered.  If the extra data is not needed, the result of this
+ *! function can be treated as a logical value.
+ */
+static void gz_end_of_stream(INT32 args)
+{
+  struct zipper *this=THIS;
+  pop_n_elems(args);
+  if(this->epilogue)
+    ref_push_string(this->epilogue);
+  else
+    push_int(0);
+}
+
 static void init_gz_inflate(struct object *o)
 {
   mt_init(& THIS->lock);
@@ -458,12 +575,14 @@ static void init_gz_inflate(struct object *o)
   THIS->gz.opaque=(void *)THIS;
   inflateInit(&THIS->gz);
   inflateEnd(&THIS->gz);
+  THIS->epilogue = NULL;
 }
 
 static void exit_gz_inflate(struct object *o)
 {
 /*   mt_lock(& THIS->lock); */
   inflateEnd(& THIS->gz);
+  do_free_string(THIS->epilogue);
 /*   mt_unlock(& THIS->lock); */
   mt_destroy( & THIS->lock );
 }
@@ -473,7 +592,7 @@ static void exit_gz_inflate(struct object *o)
 
 /*! @decl int crc32(string data, void|int start_value)
  *!
- *! This function calculates the standard ISO3309 Cyclic Redundancy Check.
+ *!   This function calculates the standard ISO3309 Cyclic Redundancy Check.
  */
 static void gz_crc32(INT32 args)
 {
@@ -498,21 +617,20 @@ static void gz_crc32(INT32 args)
    push_int((INT32)crc);
 }
 
-
 /*! @endmodule
  */
 #endif
 
-void pike_module_exit(void) {}
+PIKE_MODULE_EXIT {}
 
-void pike_module_init(void)
+PIKE_MODULE_INIT
 {
 #ifdef HAVE_ZLIB_H
   start_new_program();
   ADD_STORAGE(struct zipper);
   
-  /* function(int|void:void) */
-  ADD_FUNCTION("create",gz_deflate_create,tFunc(tOr(tInt,tVoid),tVoid),0);
+  /* function(int|void,int|void:void) */
+  ADD_FUNCTION("create",gz_deflate_create,tFunc(tOr(tInt,tVoid) tOr(tInt,tVoid),tVoid),0);
   /* function(string,int|void:string) */
   ADD_FUNCTION("deflate",gz_deflate,tFunc(tStr tOr(tInt,tVoid),tStr),0);
 
@@ -520,6 +638,9 @@ void pike_module_init(void)
   add_integer_constant("PARTIAL_FLUSH",Z_PARTIAL_FLUSH,0);
   add_integer_constant("SYNC_FLUSH",Z_SYNC_FLUSH,0);
   add_integer_constant("FINISH",Z_FINISH,0);
+  add_integer_constant("DEFAULT_STRATEGY", Z_DEFAULT_STRATEGY,0);
+  add_integer_constant("FILTERED", Z_FILTERED,0);
+  add_integer_constant("HUFFMAN_ONLY", Z_HUFFMAN_ONLY,0);
 
   set_init_callback(init_gz_deflate);
   set_exit_callback(exit_gz_deflate);
@@ -533,6 +654,8 @@ void pike_module_init(void)
   ADD_FUNCTION("create",gz_inflate_create,tFunc(tOr(tInt,tVoid),tVoid),0);
   /* function(string:string) */
   ADD_FUNCTION("inflate",gz_inflate,tFunc(tStr,tStr),0);
+  /* function(:string) */
+  ADD_FUNCTION("end_of_stream",gz_end_of_stream,tFunc(tNone,tStr),0);
 
   add_integer_constant("NO_FLUSH",Z_NO_FLUSH,0);
   add_integer_constant("PARTIAL_FLUSH",Z_PARTIAL_FLUSH,0);
@@ -548,6 +671,9 @@ void pike_module_init(void)
   add_integer_constant("PARTIAL_FLUSH",Z_PARTIAL_FLUSH,0);
   add_integer_constant("SYNC_FLUSH",Z_SYNC_FLUSH,0);
   add_integer_constant("FINISH",Z_FINISH,0);
+  add_integer_constant("DEFAULT_STRATEGY", Z_DEFAULT_STRATEGY,0);
+  add_integer_constant("FILTERED", Z_FILTERED,0);
+  add_integer_constant("HUFFMAN_ONLY", Z_HUFFMAN_ONLY,0);
 
   /* function(string,void|int:int) */
   ADD_FUNCTION("crc32",gz_crc32,tFunc(tStr tOr(tVoid,tInt),tInt),
