@@ -1,11 +1,16 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: polyfill.c,v 1.45 2004/03/05 23:04:03 nilsson Exp $
+*/
+
 #include "global.h"
-RCSID("$Id: polyfill.c,v 1.36 2000/12/01 08:10:02 hubbe Exp $");
+RCSID("$Id: polyfill.c,v 1.45 2004/03/05 23:04:03 nilsson Exp $");
 
 /* Prototypes are needed for these */
 extern double floor(double);
 
-#include "global.h"
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -14,20 +19,14 @@ extern double floor(double);
 
 #include "image_machine.h"
 
-#include "stralloc.h"
-#include "pike_macros.h"
-#include "object.h"
-#include "constants.h"
 #include "interpret.h"
 #include "svalue.h"
-#include "array.h"
 #include "threads.h"
-#include "builtin_functions.h"
 
 #include "image.h"
 
-/* This must be included last! */
-#include "module_magic.h"
+
+#define sp Pike_sp
 
 #ifdef THIS
 #undef THIS
@@ -39,8 +38,6 @@ extern double floor(double);
 
 /*
 **! module Image
-**! note
-**!	$Id: polyfill.c,v 1.36 2000/12/01 08:10:02 hubbe Exp $
 **! class Image
 */
 
@@ -56,6 +53,16 @@ extern double floor(double);
 **!	If any given curve is inside another, it
 **!	will make a hole.
 **!
+**! <pre>
+**! Image.Image(100,100)->setcolor(255,0,0,128)->
+**!   polyfill( ({ 20,20, 80,20, 80,80 }) );
+**! </pre>
+**! <illustration>
+**! return Image.Image(100,100)->setcolor(255,0,0,128)->
+**!   polyfill( ({ 20,20, 80,20, 80,80 }) );
+**! </illustration>
+**!
+**!
 **! note
 **!	Lines in the polygon may <i>not</i> be crossed without
 **!	the crossing coordinate specified in both lines.
@@ -390,10 +397,10 @@ static INLINE void polyfill_slant_add(double *buf,
 
 static int polyfill_event(double xmin,
 			  double xmax,
-			  struct line_list **pll,
-			  int tog,
 			  double yp,
-			  double *buf)
+			  double *buf,
+			  struct line_list **pll,
+			  int tog)
 {
    struct line_list *c;
    struct line_list *ll=*pll;
@@ -520,9 +527,9 @@ static void polyfill_some(struct image *img,
       double yp = y;
       struct line_list *c;
       double xmin, xmax;
+      rgb_group *d;
       int tog=0;
       int i;
-      rgb_group *d;
 
 #ifdef POLYDEBUG      
       fprintf(stderr,"\nline %d..%d\n",y,y+1);
@@ -579,7 +586,6 @@ static void polyfill_some(struct image *img,
 	       VY(c,c->xmin)>VY(c->next,c->xmin)))
 	 {
 	    struct line_list *l1;
-	    struct line_list *pll;
 	    /* resort */
 #ifdef POLYDEBUG
 	    fprintf(stderr,"  !!! resort !!!\n");
@@ -624,7 +630,7 @@ static void polyfill_some(struct image *img,
 	 if (xmax==1e10) break; /* no more events */
 
 	 if (xmax>ixmax) xmax=ixmax;
-	 tog=polyfill_event(xmin,xmax,&ll,tog,yp,buf);
+	 tog=polyfill_event(xmin,xmax,yp,buf,&ll,tog);
 	 
 	 /* shift to get next event */
 	 xmin = xmax;
@@ -642,19 +648,50 @@ static void polyfill_some(struct image *img,
 
       /* write this row */
       d=img->img+img->xsize*y;
-      for (i=0; i<img->xsize; i++)
+      if(THIS->alpha)
       {
+	for (i=0; i<img->xsize; i++)
+        {
 #ifdef POLYDEBUG
-	 fprintf(stderr,"%3.2f ",buf[i]);
+	  fprintf(stderr,"%3.2f ",buf[i]);
+#endif
+
+#define apply_alpha(x,y,alpha) \
+   ((unsigned char)((y*(255L-(alpha))+x*(alpha))/255L))
+
+	  d->r = apply_alpha( d->r,
+			      DOUBLE_TO_COLORTYPE((d->r*(1.0-buf[i]))+
+						  (img->rgb.r*buf[i])),
+			      THIS->alpha );
+	  d->g = apply_alpha( d->g,
+			      DOUBLE_TO_COLORTYPE((d->g*(1.0-buf[i]))+
+						  (img->rgb.g*buf[i])),
+			      THIS->alpha );
+	  d->b = apply_alpha( d->b,
+			      DOUBLE_TO_COLORTYPE((d->b*(1.0-buf[i]))+
+						  (img->rgb.b*buf[i])),
+			      THIS->alpha );
+	  d++;
+	}
+#ifdef POLYDEBUG
+	fprintf(stderr,"\n");
 #endif
-	 d->r = DOUBLE_TO_COLORTYPE((d->r*(1.0-buf[i]))+(img->rgb.r*buf[i]));
-	 d->g = DOUBLE_TO_COLORTYPE((d->g*(1.0-buf[i]))+(img->rgb.g*buf[i]));
-	 d->b = DOUBLE_TO_COLORTYPE((d->b*(1.0-buf[i]))+(img->rgb.b*buf[i]));
-	 d++;
       }
+      else {
+	for (i=0; i<img->xsize; i++)
+        {
 #ifdef POLYDEBUG
-      fprintf(stderr,"\n");
+	  fprintf(stderr,"%3.2f ",buf[i]);
 #endif
+	  d->r = DOUBLE_TO_COLORTYPE((d->r*(1.0-buf[i]))+(img->rgb.r*buf[i]));
+	  d->g = DOUBLE_TO_COLORTYPE((d->g*(1.0-buf[i]))+(img->rgb.g*buf[i]));
+	  d->b = DOUBLE_TO_COLORTYPE((d->b*(1.0-buf[i]))+(img->rgb.b*buf[i]));
+	  d++;
+	}
+#ifdef POLYDEBUG
+	fprintf(stderr,"\n");
+#endif
+      }
 
       y++;
    }
@@ -797,4 +834,3 @@ void image_polyfill(INT32 args)
 
    ref_push_object(THISOBJ);
 }
-
