@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: svalue.c,v 1.195 2005/05/18 12:36:54 mast Exp $
+|| $Id: svalue.c,v 1.197 2005/09/09 15:09:09 mast Exp $
 */
 
 #include "global.h"
@@ -30,7 +30,7 @@
 
 #define sp Pike_sp
 
-RCSID("$Id: svalue.c,v 1.195 2005/05/18 12:36:54 mast Exp $");
+RCSID("$Id: svalue.c,v 1.197 2005/09/09 15:09:09 mast Exp $");
 
 struct svalue dest_ob_zero = {
   T_INT, 0,
@@ -1134,6 +1134,10 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 {
   char buf[50];
 
+  /* This needs to be a bit lower than LOW_C_STACK_MARGIN so that the
+   * the raw error can be printed in exit_on_error. */
+  check_c_stack(250);
+
   check_type(s->type);
   check_refs(s);
 
